/*
    Magic eye demo.
    Copyright (C) 2017  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "magiceye/magiceye.h"			// magic eye control

extern MagicEye_t	FirstEye;		// magic eye descriptor
extern MagicEye_t	SecondEye;		// magic eye descriptor

#include "loadpng.h"				// png helper
#include "random.h"				// random generator
#include "fluid.h"				// fluid movement filter

#define ENABLE_DEBUG		1		// enabled if 1
#include "macros.h"				// debug macro

/***************************************************************************/
/* define vari */

#define UPDATE_TIME	50			// refresh ogni 50ms
#define VIDEO_W		320
#define VIDEO_H		240
#define VIDEO_MODE	(SDL_HWSURFACE|SDL_DOUBLEBUF)

/***************************************************************************/
/* strutture */

typedef struct
{
  int		Delta;		// ampiezza movimento
  int		Steps;		// durata del movimento
}Move_t;

/***************************************************************************/
/* costanti */

static const char	Licenza[]=
	"demo version 0.02, Copyright (C) 2017  Alessandro Accattini\n";
static const char	Help[]=
	"Usage: demo [OPTIONS]\n"
	"\t-a ALPHA\tset the magic eye alpha level (default=255).\n"
	"\t-h\tdisplays this help.\n"
	"\t-t TIME\tchange the refresh interval (default=100).\n"
	"\nReport bugs to: <bit.trapper@gmail.com>\n";
static const Move_t	MoveList[]={	
				{ 100,FLUID_FILTER_STEPS-15},
				{ -50,FLUID_FILTER_STEPS-10},
				{ -50,FLUID_FILTER_STEPS-20},
				{  30,FLUID_FILTER_STEPS-15},
				{ 100,FLUID_FILTER_STEPS-10},
				{ -30,FLUID_FILTER_STEPS-20},
				{  30,FLUID_FILTER_STEPS-20},
				{  30,FLUID_FILTER_STEPS-20},
				{  30,FLUID_FILTER_STEPS-15},
				{  40,FLUID_FILTER_STEPS-20},
				{ -70,FLUID_FILTER_STEPS-15},
				{0,0}
};

/***************************************************************************/
/* variabili */

volatile unsigned char	ReadTrigger=0;		// is time to read data
unsigned char		Run=1;			// run/exit flag
SDL_Surface		*Video;			// finestra base
SDL_Surface		*BackGround;		// sfondo
SDL_Surface		*Magic;			// magic eye
SDL_Surface		*Magic2;		// magic eye
int			Rotate[3]={0,0,0};	// array rotazioni
int			Rotate2[2]={0,0};	// array rotazioni

/***************************************************************************/
/* prototipi funzioni */

Uint32 TimerCallback(Uint32 Interval,void *Param);
  /* Funzione di callback del timer, esegue la lettura dei registri nel
     device remoto. */

/***************************************************************************/
/* funzioni */

Uint32 TimerCallback(Uint32 Interval,void *Param)
{
  /* Funzione di callback del timer, esegue la lettura dei registri nel
     device remoto. */
  


  static const Move_t	*Entry=MoveList;	// movement entry ptr
  static FluidFilter_t	FluidFilt;		// filtro movimento
  static FluidData_t	Pos=0;			// movimento rigido
  static FluidData_t	OldPos;
  static int		RingCount=0;		// contatore steps
  static SDL_Rect	EyePos= {.x=147-64,.y=120-64};
  static SDL_Rect	EyePos2={.x= 31-24,.y=143-24};



  // copia il background nella finestra finale
  SDL_BlitSurface(BackGround,NULL,Video,NULL);

  // anima il primo magic eye

  // primo anello
  if(!RingCount)				// se la fase e' conclusa
  {
    // inizia una nuova fase del movimento
    ResetFluidFilter(&FluidFilt);		// resetta il filtro
    OldPos=Rotate[0];				// salva l'old
    Pos=	Entry->Delta;			// movimento rigido relativo
    RingCount=	Entry->Steps;			// durata del movimento
    Entry++;					// next entry
    if(Entry->Delta == 0)			// se e' la fine della lista
      Entry=MoveList;				// riparte da capo
  }
  RingCount--;					// scala il contatore
  Rotate[0]=OldPos + FluidFilter_1(&FluidFilt,Pos);	// movimento fluido
//  if(Rotate[0] >= 128)
//    Rotate[0]-=128;

  // secondo anello
  Rotate[1]--;					// ruota verso sinistra
  if(Rotate[1] < 0)
    Rotate[1]=127;
  RandomSector(&FirstEye,1);			// update random shape
  
  // terzo anello
  Rotate[2]++;					// ruota verso destra
  if(Rotate[2] >= 128)
    Rotate[2]-=128;
  
  // anima il secondo magic eye

  // primo anello
  Rotate2[0]++;					// ruota verso destra
  if(Rotate2[0] >= 64)
    Rotate2[0]=0;

  // secondo anello
  Rotate2[1]--;					// ruota verso sinistra
  if(Rotate2[1] < 0)
    Rotate2[1]=64-1;
 
  // aggiorna i magic eyes
  MagicEye_Perspection(&FirstEye ,Rotate ,Magic );
  MagicEye_Perspection(&SecondEye,Rotate2,Magic2);
  SDL_BlitSurface(Magic ,NULL,Video,&EyePos );	// sovrappone il magic eye
  SDL_BlitSurface(Magic2,NULL,Video,&EyePos2);	// sovrappone il magic eye
  
  // aggiorna la finestra
  SDL_Flip(Video);
  return(Interval);			// termina
}





/***************************************************************************/

int main(int argc,char **argv)
{
  SDL_TimerID		SerialTimer;		// timer lettura registri
  SDL_Event		InputEvent;		// evento in ingresso
  unsigned long int	ReadInterval=UPDATE_TIME;
  int			Opzione;		// per la lettura delle opzioni
  int			i;
  unsigned char		Alpha=255;		// magic eye alpha level



  // visualizza la licenza del programma
  puts(Licenza);

  // legge le opzioni presenti sulla riga di comando
  while((Opzione=getopt(argc,argv,"ht:a:"))!=-1)
  {
    switch(Opzione)
    {
      case 'h':
        puts(Help);			// stampa l'help
	return 0;			// termina
	break;
      case 't':
        sscanf(optarg,"%li",&ReadInterval);
	break;
      case 'a':
        sscanf(optarg,"%hhu",&Alpha);
	break;
      default:
        puts("Opzione sconosciuta !!!");
	exit(0);				// termina
	break;
    }
  }

  // inizializza la SDL
  PDEBUG("%s: initializing SDL...\n",__FUNCTION__);
  SDL_putenv("SDL_VIDEO_X11_WMCLASS=littlegptracker");
  if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER)!=0)
  {
    /* termina causa errore di inizializzazione */
    fprintf(stderr,"SDL_Init: %s\n",SDL_GetError());
    return 1;					// termina con errore
  }
  atexit(SDL_Quit);				// registra la SDL_Quit()

  // crea la finestra base
  if((Video=SDL_SetVideoMode(VIDEO_W,VIDEO_H,16,VIDEO_MODE))==NULL)
  {
    // termina causa errore creazione finestra
    fprintf(stderr,"SDL_SetVideoMode: %s\n",SDL_GetError());
    return 1;					// termina con fallimento
  }
  
  // carica tutte le immagini da disco
  LoadSetVideo(Video);				// finestra di riferimento
  if((BackGround=Load16bitPng("eye_2.png")) == NULL)
    puts("Unable to load background");

  // crea il magic eye
  Magic=MagicEye_CreateSurface(&FirstEye);		// crea la surface
  SDL_SetColorKey(Magic,SDL_SRCCOLORKEY,0x0000);	// nero trsap.
  SDL_SetAlpha(Magic,SDL_SRCALPHA,Alpha);		// alpha level

  Magic2=MagicEye_CreateSurface(&SecondEye);		// crea la surface
  SDL_SetColorKey(Magic2,SDL_SRCCOLORKEY,0x0000);	// nero trsap.
  SDL_SetAlpha(Magic2,SDL_SRCALPHA,Alpha);		// alpha level
  
  // colora il primo magic eye
  MagicEye_Fill(&FirstEye,0x0000);			// tutto nero
  
  // primo anello
  for(i=0;i<32;i++)
    MagicEye_PutPixel(&FirstEye,i,0,0xffff);		// init. del bianco
  for(i=64;i<80;i++)
    MagicEye_PutPixel(&FirstEye,i,0,0xffff);		// init. del bianco
  
  // secondo anello
  for(i=32;i<40;i++)
    MagicEye_PutPixel(&FirstEye,i,1,0xffff);		// init. del bianco
  for(i=120;i<128;i++)
    MagicEye_PutPixel(&FirstEye,i,1,0xffff);		// init. del bianco
  RandomSector(&FirstEye,1);				// random shape
  
  // terzo anello
  for(i=0;i<26;i++)
    MagicEye_PutPixel(&FirstEye,i,2,0xffff);		// init. del bianco
  for(i=72;i<88;i++)
    MagicEye_PutPixel(&FirstEye,i,2,0xffff);		// init. del bianco
  
  // colora il secondo magic eye
  MagicEye_Fill(&SecondEye,0x0000);			// tutto nero
  
  for(i=0;i<32;i++)
    MagicEye_PutPixel(&SecondEye,i,0,0xffff);		// init. del bianco
  for(i=0;i<20;i++)
    MagicEye_PutPixel(&SecondEye,i,1,0xffff);		// init. del bianco

  // crea il timer aggiornamento video
  PDEBUG("%s: creating screen update timer...\n",__FUNCTION__);
  if((SerialTimer=SDL_AddTimer(ReadInterval,TimerCallback,NULL))==NULL)
    fprintf(stderr,"SDL_AddTimer: %s\n",SDL_GetError());

  // stampa le istruzioni
  puts("Press 'q' to exit.");



  // main loop
  while(Run)
  {
    usleep(20000);				// 20ms
    if(!SDL_PollEvent(&InputEvent))		// se non succede niente
      continue;					// fine del discorso

    switch(InputEvent.type)
    {
//      case SDL_MOUSEBUTTONDOWN:
//      case SDL_MOUSEBUTTONUP:
//      case SDL_MOUSEMOTION:
      case SDL_QUIT:
	Run=0;					// abbandona il loop
	break;
      case SDL_KEYDOWN:
        switch(InputEvent.key.keysym.sym)
	{
	  case SDLK_q:
	    Run=0;				// abbandona il loop
	    break;
/*	    
	  case SDLK_z:
	    Rotate[0]--;
	    printf("Rotate[0]=%i\n",Rotate[0]);
	    break;
	  case SDLK_x:
	    Rotate[0]++;
	    printf("Rotate[0]=%i\n",Rotate[0]);
	    break;
	  case SDLK_n:
	    Rotate[1]--;
	    printf("Rotate[1]=%i\n",Rotate[1]);
	    break;
	  case SDLK_m:
	    Rotate[1]++;
	    printf("Rotate[1]=%i\n",Rotate[1]);
	    break;
*/	    
	}
	break;
    }
  }

  // libera le risorse allocate
  puts("Exiting...");
  SDL_RemoveTimer(SerialTimer);			// rimuove il timer
  SDL_FreeSurface(Magic);			// dealloca le surface
  SDL_FreeSurface(Magic2);
  SDL_FreeSurface(BackGround);			// dealloca il background
  SDL_FreeSurface(Video);			// dealloca il video
  return 0;					// termina
}

/* $Source$ */

