/*
    Fluid movement digital filter implementation.
    Copyright (C) 2017  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __FLUID_C
#include "fluid.h"				// functions prototypes

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

void ResetFluidFilter(FluidFilter_t *Filter)
{
  /* Resetta gli stati interni del filtro. */



  Filter->In[0]=0.0;
  Filter->In[1]=0.0;
  Filter->Out[0]=0.0;
  Filter->Out[1]=0.0;
  return;				// termina
}



FluidData_t FluidFilter_1(FluidFilter_t *Filter,FluidData_t Input)
{
  /* Fluid digital filter: smussa il movimento senza sovraelongazione. */

  /*
     y(n)=(x(n) + x(n-2)) / 100 + (p1+p2) y(n-1) - p1p2 y(n-2)

     con: p1=p2=0.86
  */



  Filter->In[2]=Filter->In[1];		// shift degli ingressi
  Filter->In[1]=Filter->In[0];
  Filter->In[0]=(float)Input;
  Filter->Out[2]=Filter->Out[1];	// shift delle uscite
  Filter->Out[1]=Filter->Out[0];
  Filter->Out[0]=	(Filter->In[0] + Filter->In[2]) / 100.0 + 
  		(0.86+0.86)*Filter->Out[1] - (0.86*0.86)*Filter->Out[2];
  return(Filter->Out[0]);
}



FluidData_t FluidFilter_2(FluidFilter_t *Filter,FluidData_t Input)
{
  /* Fluid digital filter: smussa il movimento con sovraelongazione. */
  
  /*
     y(n)=(x(n) + x(n-2)) / 80 + 2r cos(fi) y(n-1) - r**2 y(n-2)

     con: r=0.92   cos(fi)=0.99

  */



  Filter->In[2]=Filter->In[1];		// shift degli ingressi
  Filter->In[1]=Filter->In[0];
  Filter->In[0]=(float)Input;
  Filter->Out[2]=Filter->Out[1];	// shift delle uscite
  Filter->Out[1]=Filter->Out[0];
  Filter->Out[0]=	(Filter->In[0] + Filter->In[2]) / 80.0 + 
  		(2.0*0.92*0.99)*Filter->Out[1] - (0.99*0.99)*Filter->Out[2];
  return(Filter->Out[0]);
}

/* $Source$ */

