/*
    Image loading helper.
    Copyright (C) 2012 2013 2014  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __LOADPNG_C
#include "loadpng.h"			// functions prototypes

#include <SDL/SDL_image.h>
#include <stdio.h>

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static SDL_Surface	*Video=NULL;		// video utilizzato

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

SDL_Surface *LoadSetVideo(SDL_Surface *DecVideo)
{
  /* Dichiara quale SDL_Surface utilizzare come riferimento per caricare
     le immagini. */



  Video=DecVideo;			// salva il video utilizzato
  return(DecVideo);			// termina
}



SDL_Surface *LoadNewSurface(int w,int h)
{
  /* Semplifica la chiamata alla funzione SDL_CreateRGBSurface()
     richiedendo molti meno parametri. */
  


  if(Video == NULL)			// se non abbiamo un video
    return(NULL);			// fanculo tutti
  return(SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_ASYNCBLIT,
  			w,h,16,
			Video->format->Rmask,
			Video->format->Gmask,
			Video->format->Bmask,
			Video->format->Amask));	// termina
}



SDL_Surface *Load16bitPng(const char *FileName)
{
  /* Carica l'immagine specificata in una surface a 16 bit. In caso di
     successo ritorna un puntatore alla nuova surface, NULL altrimenti. */
  
  
  
  SDL_Surface	*LoadImg=NULL;		// immagine caricata da disco
  SDL_Surface	*Img16bit;		// immagine a 16 bit


  
  if((LoadImg=IMG_Load(FileName))==NULL)
  {
    fprintf(stderr,"IMG_Load() failed\n");
    return NULL;			// termina con fallimento
  }
  if((Img16bit=LoadNewSurface(LoadImg->w,LoadImg->h)) == NULL)
  {
    fprintf(stderr,"%s: LayerNewSurface() failed\n",__FUNCTION__);
    SDL_FreeSurface(LoadImg);		// distrugge l'immagine originale
    return(NULL);			// termina con fallimento
  }
  SDL_BlitSurface(LoadImg,NULL,Img16bit,NULL);	// copia l'immagine a 16 bit
  SDL_FreeSurface(LoadImg);		// distrugge l'immagine originale
  return(Img16bit);			// termina con successo
}



int Load16bitTiles(const char *FileName,int GeoX,int GeoY,SDL_Surface **List)
{
  /* Carica un'immagine composta da GeoX x GeoY tile e suddivide le tessere
     negli elementi di List, List deve avere GeoX X GoeY elementi.
     Ritorna 0 in caso di successo. */
  
  
  
  SDL_Surface	*LoadImg=NULL;		// immagine caricata da disco
  SDL_Rect	Src;			// pezzo sorgente
  int		Dx,Dy;			// dimensione originale
  int		i,j;
  
  
  
  // carica l'immagine intera
  if((LoadImg=IMG_Load(FileName))==NULL)
  {
    fprintf(stderr,"IMG_Load() failed\n");
    return 1;				// termina con fallimento
  }

  // determina la dimensione di ogni singola tessera
  Src.w=LoadImg->w / GeoX;		// dimensione tessere
  Src.h=LoadImg->h / GeoY;
  
  for(j=0;j<GeoY;j++)
  {
    Src.y=j * Src.h;			// riga da cui estrarre il tile
    for(i=0;i<GeoX;i++)
    {
      Src.x=i * Src.w;			// colonna da cui estrarre il tile
      *List=LoadNewSurface(Src.w,Src.h);		// nuova surface
      SDL_BlitSurface(LoadImg,&Src,*List++,NULL);	// la riempie
    }
  }
  SDL_FreeSurface(LoadImg);		// distrugge l'immagine originale
  return 0;				// termina con successo
}



void TilePaste(SDL_Surface *Tile,SDL_Surface *Dest,int x,int y)
{
  /* Copia la tessera Tile all'interno della surface Dest, le coordinate della
     destinazione sono espresse in multipli della dimensione della tessera. */
  
  
  
  SDL_Rect	Pos;			// posizione finale



  Pos.x=Tile->w * x;			// determina la posizione in pixel
  Pos.y=Tile->h * y;
  SDL_BlitSurface(Tile,NULL,Dest,&Pos);	// posiziona la tessera
  return;				// termina
}

/* $Source$ */

